#!/bin/sh
# $Id: t-octas-se-1.sh,v 1.5 2011/05/20 20:24:43 ca Exp $
#
# ----------------------------------------
# test outgoing restriction: transactions per session
# ----------------------------------------
#

if test "X"${MTA_TIMING} = "X"
then
  echo "SKIPPED: $0: MTA_TIMING is not set"
  exit 0
fi

SD=`dirname $0`
if test "${SD}" = "."
then
   SD=../chkmts
fi
. ${SD}/common.sh

if test X"`uname`" != "XOSF1"
then
while getopts s:V FLAG
do
  case "${FLAG}" in
    s) SKIP="${SKIP} ${OPTARG}";;
    V) VERBOSE=true;;
  esac
done
shift `expr ${OPTIND} - 1`
fi

# cd to right directory (where the programs will be executed).
cd ../qmgr || exit 1

# create directory to pass socket
test -d smtps || mkdir smtps

killit()
{
rm -f ${QMGRCONFMAP} ${MTMAP}
exit 1
}

# currently no map data needed/used
${CAT} > ${QMGRCONF} <<EOF
smtpc_conf_rcpt_domain:dom1.tld	connection_limits=cl1;
smtpc_conf_rcpt_domain:.dom1.tld	connection_limits=cl2;
EOF
# create map
rm -f ${QMGRCONFMAP}
${MM} -t'	' -F ${QMGRCONFMAP} < ${QMGRCONF}
rm -f ${MTMAP}
trap killit 2

# ----------------------------------------
${CAT} >${SMXCNF} <<EOF
smtps { log_level = 14; pass_fd_socket = smtps/smtpsfd; wait_for_server=4;
  listen_socket { type=inet; port=${SRVPORT}; }
  start_action = pass; user = ${USER};
  path = ../smtps/smtps;
  arguments = "smtps -f ${SMXCNF} ${MTA_SERVER_OPTIONS}";
}
smtpc { log_level = 14; remote_port = ${SNKPORT}; wait_for_server=4;
  connect_only_to = 127.0.0.1;
  io_timeout=60s;
  start_action = wait; user = ${USER};
  path = "../smtpc/smtpc"; arguments = "smtpc -f ${SMXCNF}";
}
qmgr { log_level = 14; wait_for_server=4; wait_for_client=4; 
  start_action = wait; user = ${USER};
  flags = { reuse_connection};
  smtpc { transactions_per_session_max=10; connections_max=100;
    connection_limits cl1 {
      transactions_per_session_max = 2;
    }
    connection_limits cl2 {
      transactions_per_session_max = 4;
    }
  };
  path = "../qmgr/qmgr"; arguments = "qmgr -f ${SMXCNF} -d 2";
}
smar { log_level = 14;
  start_action = wait; user = ${USER};
  path = "../smar/smar"; arguments = "smar -f ${SMXCNF}";
}
EOF

${CAT} > ${MT} <<EOF
b.c	[10.2.1.1]
dom1.tld	[10.1.1.1]
sub.dom1.tld	[10.1.1.2]
EOF

clearlogs

# ----------------------------------------
# 
TEST=1
if echo "${SKIP} X" | grep -v " ${TEST} " > /dev/null
then
ERR=false
# 
Nmsgs=50
if ${SHELL} ${SD}/mcpmts.sh -1 -A $Nmsgs -E $Nmsgs -Zsmtpc2 -t $Nmsgs -c "-m $Nmsgs -s $Nmsgs" -R"r@dom1.tld"
then
  if grep 'Max TAs per session * [12]$' ${SRVL} >/dev/null
  then
    :
  else
    ERR=true
    echo "$0: test ${TEST}: missing Max TAs per session 2"
    test X"${MTA_STOPONERROR}" != X && exit 1
  fi
else
  ERR=true
fi
if ${ERR}
then
  ERRS=`expr ${ERRS} + 1 `
  echo "$0: test ${TEST} failed"
  test X"${MTA_STOPONERROR}" != X && exit 1
else
  ${VERBOSE} && echo "test ${TEST} succeeded"
fi
fi

# ----------------------------------------
# 
TEST=2
if echo "${SKIP} X" | grep -v " ${TEST} " > /dev/null
then
ERR=false
# 
Nmsgs=50
if ${SHELL} ${SD}/mcpmts.sh -1 -A $Nmsgs -E $Nmsgs -Zsmtpc2 -t $Nmsgs -c "-m $Nmsgs -s $Nmsgs" -R"r@sub.dom1.tld"
then
  if grep 'Max TAs per session * [1234]$' ${SRVL} >/dev/null
  then
    :
  else
    ERR=true
    echo "$0: test ${TEST}: missing Max TAs per session 2"
    test X"${MTA_STOPONERROR}" != X && exit 1
  fi
else
  ERR=true
fi
if ${ERR}
then
  ERRS=`expr ${ERRS} + 1 `
  echo "$0: test ${TEST} failed"
  test X"${MTA_STOPONERROR}" != X && exit 1
else
  ${VERBOSE} && echo "test ${TEST} succeeded"
fi
fi

rm -f ${QMGRCONFMAP} f ${MTMAP}

# ----------------------------------------
# end
if test ${ERRS} -eq 0
then
  exit 0
else
  echo "$0: ${ERRS} error(s)"
  exit 1
fi
