#!/bin/sh
# $Id: t-verp-1.sh,v 1.8 2011/01/10 17:36:50 ca Exp $
#
# ----------------------------------------
# test VERP handling, options set in config file
# uses smtpcnf.sh
# ----------------------------------------
#

SD=`dirname $0`
if test "${SD}" = "."
then
   SD=../chkmts
fi
. ${SD}/common.sh

OUT="t-list-0.out"

if test X"`uname`" != "XOSF1"
then
while getopts s:V FLAG
do
  case "${FLAG}" in
    s) SKIP="${SKIP} ${OPTARG}";;
    V) VERBOSE=true;;
  esac
done
shift `expr ${OPTIND} - 1`
fi

# cd to working directory for alias file!
cd ../qmgr || exit 1

# create aliases
${CAT} > ${ALI} <<EOF
abuse:	local:
postmaster:	local:
user:	local:
user++:	local:
sender: local:
list0-user1: local:
list0-user2: local:
list3-user1: local:
list3-user2: local:
listl-user1: local:
listl-user2: local:
ownerlist0: local:
ownerlist1: local:
ownerlistl: local:
list0:     <list0-user1@local.dom> <list1@local.dom> <list0-user2@some.dom>
verp-list0:     ownerlist0
verp-list1:     ownerlist1
list1:     <list1-user1@remote.dom> <list1-user2@some.dom>
verp-list2@some.dom:     <ownerlist2somedom@some.dom>
list2@some.dom:     <list0@local.dom> <list1@local.dom> <list2-user0@some.dom>
verp-listl:     ownerlistl
listl:     <listl-user1@local.dom> <listl-user2@local.dom>
list3:     <list3-user1@local.dom> <list3-user2@some.dom>
verp-list3:     ownerlist3
list4:     <list4-user1@local.dom> <list0-user2@local.dom>
verp-list4:     ownerlist4
list4-user1: local:
EOF
# create map
rm -f ${ALIMAP}
${MM} < ${ALI}

# create mailertable (save old version?)
${CAT} > ${MT} <<EOF
local.dom	lmtp:
other.dom	[127.0.0.1]
remote.dom	[127.0.0.1]
some.dom	[127.0.0.1]
EOF

# ----------------------------------------
# test: send mail to local list, check Return-Path: for specified delimiters
TEST=1
if echo " ${SKIP} X" | ${GREP} -v " ${TEST} " > /dev/null
then
ERR=false

# ----------------------------------------
${CAT} > ${SMXCNF} <<EOF
qmgr { Log_Level = 12; wait_for_server=4; wait_for_client=4;
VERP { delimiter1 = /; delimiter2 = _; }
}
smar {
# aliases { flags = local_domains; }
Log_Level = 12;
nameserver = ${NS};
DNS_timeout = 5;
}
smtpc { Log_Level = 12; remote_port=${SNKPORT}; wait_for_server=4; }
smtps {
Log_Level = 14;
daemon_address = localhost:${SRVPORT};
wait_for_server=4;
}
EOF

if ${SHELL} ${SD}/smtpcnf.sh -3 lmtpsock -Ssender@local.dom -Rlistl@local.dom -E 0 -X 0 -A 1
then
  # on some systems the RPCTs may have been delivered in "reverse" order
  # hence need to check both.  it would be simpler to use:
  # ${GREP} '^Return-Path: <owner-listl/listl-user[12]_local\.dom@local\.dom>'
  # but then (theoretically) both files could contain the same sender...
  if test -s ${B1}
  then
    if ${GREP} '^Return-Path: <owner-listl/listl-user1_local\.dom@local\.dom>' ${B1} >/dev/null
    then
      :
    else
      if ${GREP} '^Return-Path: <owner-listl/listl-user2_local\.dom@local\.dom>' ${B1} >/dev/null
      then
        :
      else
        echo "$0: ${TEST}: ${GREP} in ${B1} failed"
        ERR=true
      fi
    fi
  else
    echo "$0: ${TEST}: ${B1} not found"
    ERR=true
  fi
  if test -s ${B2}
  then
    if ${GREP} '^Return-Path: <owner-listl/listl-user2_local\.dom@local\.dom>' ${B2} >/dev/null
    then
      :
    else
      if ${GREP} '^Return-Path: <owner-listl/listl-user1_local\.dom@local\.dom>' ${B2} >/dev/null
      then
        :
      else
        echo "$0: ${TEST}: ${GREP} in ${B2} failed"
        ERR=true
      fi
    fi
  else
    echo "$0: ${TEST}: ${B2} not found"
    ERR=true
  fi
else
  ERR=true
fi
if ${ERR}
then
   echo "$0: test ${TEST} failed"
   ERRS=`expr ${ERRS} + 1 `
   test X"${MTA_STOPONERROR}" != X && exit 1
else
   rm -f ${B1} ${B2}
fi
else
  ${VERBOSE} && echo "$0: test ${TEST} SKIPPED"
fi

# ----------------------------------------
# test: send mail to local list, check Return-Path:, VERP turned off
TEST=2
if echo " ${SKIP} X" | ${GREP} -v " ${TEST} " > /dev/null
then
ERR=false

# ----------------------------------------
${CAT} > ${SMXCNF} <<EOF
qmgr { Log_Level = 12; wait_for_server=4; wait_for_client=4;
VERP { when=never;}
}
smar {
Log_Level = 12;
nameserver = ${NS};
DNS_timeout = 5;
}
smtpc { Log_Level = 12; remote_port=${SNKPORT}; wait_for_server=4; }
smtps {
Log_Level = 14;
daemon_address = localhost:${SRVPORT};
wait_for_server=4;
}
EOF

if ${SHELL} ${SD}/smtpcnf.sh -3 lmtpsock -Ssender@local.dom -Rlist4@local.dom -E 0 -X 0 -A 1
then
  # no VERP: one transaction only!
  if test -s ${B1}
  then
    if ${GREP} '^Return-Path: .*[+=]' ${B1} >/dev/null
    then
      echo "$0: ${TEST}: ERROR: found wrong Return-Path in ${B1}"
      ERR=true
    fi
  else
    echo "$0: ${TEST}: ERROR: ${B1} not found"
    ERR=true
  fi
  if test -s ${B2}
  then
    echo "$0: ${TEST}: ERROR: ${B2} found"
    ERR=true
  fi
else
  ERR=true
fi
if ${ERR}
then
   echo "$0: test ${TEST} failed"
   ERRS=`expr ${ERRS} + 1 `
   test X"${MTA_STOPONERROR}" != X && exit 1
fi
else
  ${VERBOSE} && echo "$0: test ${TEST} SKIPPED"
fi

# ----------------------------------------
# test: send mail to local address, check Return-Path:, VERP turned on
TEST=3
if echo " ${SKIP} X" | ${GREP} -v " ${TEST} " > /dev/null
then
ERR=false

# ----------------------------------------
${CAT} > ${SMXCNF} <<EOF
qmgr { Log_Level = 12; wait_for_server=4; wait_for_client=4;
VERP { when=always;}
}
smar {
Log_Level = 12;
nameserver = ${NS};
DNS_timeout = 5;
}
smtpc { Log_Level = 12; remote_port=${SNKPORT}; wait_for_server=4; }
smtps {
Log_Level = 14;
daemon_address = localhost:${SRVPORT};
wait_for_server=4;
}
EOF

if ${SHELL} ${SD}/smtpcnf.sh -3 lmtpsock -Ssender@some.dom -Ruser@local.dom -E 0 -X 0 -A 1
then
  if test -s ${B1}
  then
    if ${GREP} '^Return-Path: <sender+user=local.dom@some.dom>' ${B1} >/dev/null
    then
      :
    else
      echo "$0: ${TEST}: ERROR: found wrong Return-Path in ${B1}"
      ERR=true
    fi
  else
    echo "$0: ${TEST}: ERROR: ${B1} not found"
    ERR=true
  fi
  if test -s ${B2}
  then
    echo "$0: ${TEST}: ERROR: ${B2} found"
    ERR=true
  fi
else
  ERR=true
fi
if ${ERR}
then
   echo "$0: test ${TEST} failed"
   ERRS=`expr ${ERRS} + 1 `
   test X"${MTA_STOPONERROR}" != X && exit 1
fi
else
  ${VERBOSE} && echo "$0: test ${TEST} SKIPPED"
fi


rm -f ${MTMAP} ${ALIMAP}
# ----------------------------------------
# end
if test ${ERRS} -eq 0
then
  exit 0
else
  echo "$0: ${ERRS} error(s)"
  exit 1
fi
