/*
 * Copyright (c) 2004, 2005 Sendmail, Inc. and its suppliers.
 *	All rights reserved.
 *
 * By using this file, you agree to the terms and conditions set
 * forth in the LICENSE file which can be found at the top level of
 * the sendmail distribution.
 */

#include "sm/generic.h"
SM_RCSID("@(#)$Id: t-conf-10.c,v 1.2 2012/11/05 15:24:57 ca Exp $")

#if SM_LIBCONF_ALONE
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "sm-conf.h"
#else /* SM_LIBCONF_ALONE */
#include "sm/string.h"
#include "sm/net.h"
#include "sm/io.h"
#include "sm/sm-conf.h"
#include "sm/sm-conf-prt.h"
#define SM_CONF_LOG_DEF 1
#include "sm/logcnfdef.h"
#include <stdio.h>
#endif /* SM_LIBCONF_ALONE */
#include "sm/sm-conf-expr.h"
#include "sm-expr.h"

/*
**  Just a test program to play around with libconf.
*/

enum dhparam_type_E
{
	DHPBITS  = 1,
	DHPFILE  = 2
};

typedef enum dhparam_type_E dhparam_type_T;
typedef union dhparam_U
{
	dhparam_type_T dhparam_type;

	struct {
	       dhparam_type_T   dhpfile_type;
	       char            *dhpfile_name;
	} dhpfile;

	struct {
	       dhparam_type_T   dhbits_type;
	       int              dhbits_bits;
	} dhbits;

} dhparam_T;

static sm_conf_definition_T const
dhbits_definitions[] =
{
	{ SM_CONF_DEF_MAGIC, "bits", sm_conf_type_u32,
		offsetof(dhparam_T, dhbits.dhbits_bits),
		sizeof(unsigned int), "1"
	},

CONF_SENTINEL
};

static sm_conf_definition_T const
dhpfile_definitions[] =
{
	{ SM_CONF_DEF_MAGIC, "file", sm_conf_type_string,
		offsetof(dhparam_T, dhpfile.dhpfile_name), 0, NULL
	},

CONF_SENTINEL
};

sm_conf_definition_T dhparam_definitions[] = {
	{ SM_CONF_DEF_MAGIC, "", sm_conf_type_union_type,
		offsetof(dhparam_T, dhparam_type),
		sizeof(dhparam_type_T)
	},
	{ SM_CONF_DEF_MAGIC, "bits", sm_conf_type_union_choice, DHPBITS,
		sizeof(dhparam_T), /* default */ NULL, /* flags */ 0,
		dhbits_definitions
	},
	{ SM_CONF_DEF_MAGIC, "file", sm_conf_type_union_choice, DHPFILE,
		sizeof(dhparam_T), /* default */ NULL, /* flags */ 0,
		dhpfile_definitions
	},
CONF_SENTINEL
};

sm_conf_definition_T
mts_def[] =
{

CONF_SENTINEL
};


typedef struct tls_cnf_S		tls_cnf_T, *tls_cnf_P;
struct tls_cnf_S
{
	char		*tlscnf_cert;
	uint		 tlscnf_cache_size;
	uint		 tlscnf_cache_tmout;
	uchar		 tlscnf_options_isset;
	dhparam_T	 tlscnf_dhparam;
};

typedef struct
{
	tls_cnf_T	cnf_tls;
} structure;

sm_conf_definition_T
tls_defs[] = {
{ SM_CONF_DEF_MAGIC, "cert_file",	sm_conf_type_string,
	offsetof(tls_cnf_T, tlscnf_cert),	0,
	"smcert.pem",
	0,	NULL,	NULL,	NULL,
	"File with certificate in PEM format"
	SM_LC_NO_ISSET SM_LC_SET_MAGIC(0) }

,{ SM_CONF_DEF_MAGIC, "cache_size",		sm_conf_type_u32,
	offsetof(tls_cnf_T, tlscnf_cache_size),	sizeof(uint),
	NULL,	0,	NULL,	NULL,	NULL,
	"size of TLS session cache"
	SM_LC_NO_ISSET
	SM_LC_SET_MAGIC(0) }

,{ SM_CONF_DEF_MAGIC, "cache_timeout",		sm_conf_type_u32,
	offsetof(tls_cnf_T, tlscnf_cache_tmout),	sizeof(uint),
	NULL,	0,	NULL,
	NULL,	NULL,
	"timeout for TLS session cache"
	SM_LC_NO_ISSET SM_LC_SET_MAGIC(0) }

,{ SM_CONF_DEF_MAGIC, "dhparam", sm_conf_type_union,
	offsetof(tls_cnf_T, tlscnf_dhparam), sizeof(dhparam_type_T),
	NULL, 0, dhparam_definitions,
	NULL,	NULL,
	"DH parameters"
	SM_LC_NO_ISSET SM_LC_SET_MAGIC(0) }

,CONF_SENTINEL
};

sm_conf_definition_T
definitions[] =
{
{ SM_CONF_DEF_MAGIC, "tls", sm_conf_type_section,
	offsetof(structure, cnf_tls), sizeof(structure), NULL,
	SM_CONF_FLAG_KEEP_DEFAULT,
	tls_defs,
	NULL, NULL, NULL
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },

CONF_SENTINEL
};

static structure s;

static int
process(char const *name, FILE *fp)
{
	sm_conf_T *stream;
	int err;
	sm_str_P text;
	char buf[SM_CONF_ERROR_BUFFER_SIZE];
	char const *e = NULL;

	if (((stream = sm_conf_new(name ? name : "*stdin*"))) == NULL) {
		fprintf(stderr, "error -- sm_conf_new() returns NULL!\n");
		return 1;
	}
	if ((err = sm_conf_read_FILE(stream, name, fp)) != 0) {
		fprintf(stderr, "%s: %s\n",
			name ? name : "*stdin*",
			sm_conf_strerror(err, buf, sizeof buf));

		while ((e = sm_conf_syntax_error(stream, e)) != NULL)
			fprintf(stderr, "%s\n", e);

		sm_conf_destroy(stream);
		return 2;
	}

	if ((err = sm_conf_scan(stream, definitions, 0, &s)) != 0) {
		fprintf(stderr, "(while scanning) %s: %s\n",
			name ? name : "*stdin*",
			sm_conf_strerror(err, buf, sizeof buf));
		while ((e = sm_conf_syntax_error(stream, e)) != NULL)
			fprintf(stderr, "%s\n", e);
		sm_conf_destroy(stream);
		return 3;
	}

	text = sm_str_new(NULL, 256, 8192);
	sm_io_fprintf(smioout, "prt_conf1:\n");
	(void) sm_conf_prt_cnfs(definitions, &s, 0, smioout, 0, NULL, text, 0);
	sm_io_flush(smioout);

	return 0;
}

int
main(int ac, char **av)
{
	int ai, c, ret;

	while ((c = getopt(ac, av, "f:")) != -1) {
		switch (c) {
		  case 'd':
			sm_conf_prt_dflt(definitions, SMC_FLD_FLAGS, smioout);
			sm_io_flush(smioout);
			break;
		  case 'f':
			ret = process(optarg, NULL);
			if (ret != 0)
				return ret;
			return 0;
			break;
		}
	}
	ac -= optind;
	av += optind;
	if (ac == 0)
		return process("*stdin*", stdin);

	for (ai = 0; ai < ac; ai++) {
		int ret = process(av[ai], NULL);
		if (ret != 0)
			return ret;
	}
	return 0;
}
