/*
 * Copyright (c) 2010 Sendmail, Inc. and its suppliers.
 *      All rights reserved.
 *
 * By using this file, you agree to the terms and conditions set
 * forth in the LICENSE file which can be found at the top level of
 * the sendmail distribution.
 *
 *	$Id: dkim.h,v 1.29 2011/01/18 21:02:28 ca Exp $
 */

#include "sm/generic.h"

#ifndef LIBDKIM_DKIM_H
#define LIBDKIM_DKIM_H 1
#if MTA_USE_DKIM
#include "sm/magic.h"
#include "sm/time.h"
#include "sm/io.h"
#include "sm/str.h"

#include "sm/dkim.h"

#define DKIM_HDRLEN	256

typedef struct dkim_cookie_S	dkim_cookie_T, *dkim_cookie_P;

/* context per DKIM "file" (header, body) */
struct dkim_cookie_S
{
	dkim_msg_P	 dkf_msg;
	bool		 dkf_sha256;
	dkim_sha_T	 dkf_sha;
};

#if 0
#define SM_IS_DKIM_CTX(dkim_ctx) SM_REQUIRE_ISA((dkim_ctx), SM_DKIM_MAGIC)
#else
#define SM_IS_DKIM_CTX(dkim_ctx) SM_REQUIRE((dkim_ctx) != NULL)
#endif

extern sm_stream_T SmDKIMStL;

#define DKIM_SHA1_CTX(dkc)	((dkc)->dkf_sha.dkim_sha1.sha1_ctx)
#define DKIM_SHA256_CTX(dkc)	((dkc)->dkf_sha.dkim_sha256.sha256_ctx)
#define DKIM_SHA1_OUT(dkc)	((dkc)->dkf_sha.dkim_sha1.sha1_out)
#define DKIM_SHA256_OUT(dkc)	((dkc)->dkf_sha.dkim_sha256.sha256_out)

sm_ret_T dkim_gensig(dkim_ctx_P _dkim_ctx, dkim_msg_P _dkim_msg, dkim_cookie_P _dkim_cookie);

/* more methods? */
#define DKIM_CANON(c)	(((c) == DKIM_CANON_SIMPLE) ? "simple" : "relaxed")
#define DKIM_C_REL(c)	((c) == DKIM_CANON_RELAXED)

#if DKIM_DEBUG
#define PRTBUF(txt, buf, len) do { int j;\
        sm_io_fprintf(smioerr, "%s\n", txt); \
        for (j = 0; j < (len); j++) \
                sm_io_fprintf(smioerr, "%02x", (((char *)(buf))[j] & 0x00ff)); \
        sm_io_fprintf(smioerr, "\n"); \
        } while (0)
#else
#define PRTBUF(txt, buf, len)
#endif

sm_ret_T dkh_new(dkhhd_P _dkhhd, dkh_P *_pdkh);
sm_ret_T dkhl_free(dkhhd_P _dkhhd);

#endif /* MTA_USE_DKIM */

#endif /* LIBDKIM_DKIM_H */
