/*
 * Copyright (c) 1990, 1993
 *	The Regents of the University of California.  All rights reserved.
 *
 * This code is derived from software contributed to Berkeley by
 * Chris Torek.
 *
 * By using this file, you agree to the terms and conditions set
 * forth in the LICENSE file which can be found at the top level of
 * the sendmail distribution.
 *
 *	$Id: print.c,v 1.15 2014/04/19 15:55:30 ca Exp $
 */

		  case 'O':
			flags |= LONGINT;
			/*FALLTHROUGH*/
		  case 'o':
			_uquad = UARG();
			base = OCT;
			goto nosign;
		  case 'p':

			/*
			**  ``The argument shall be a pointer to void.  The
			**  value of the pointer is converted to a sequence
			**  of printable characters, in an implementation-
			**  defined manner.''
			**	-- ANSI X3J11
			*/

			/* NOSTRICT */
			{
				union {
					void *p;
					ulonglong_T ll;
					ulong l;
					uint i;
				} u;
				u.p = GETARG(void *);
				if (sizeof(void *) == sizeof(ulonglong_T))
					_uquad = u.ll;
				else if (sizeof(void *) == sizeof(long))
					_uquad = u.l;
				else
					_uquad = u.i;
			}
			base = HEX;
			xdigs = "0123456789abcdef";
			flags |= HEXPREFIX;
			ch = 'x';
			goto nosign;

		  case 'H':
			_uquad = SARG();
			cp = smmod2txt((sm_ret_T) _uquad);
			if (cp == NULL)
				cp = "unknown_module";
			goto prtstring;
		  case 'M':
			_uquad = SARG();
			if ((longlong_T) _uquad >= 0) {
				base = DEC;
				goto number;
			}
			else if (prec < 0) {
				sm_error_T err;

				err = (sm_error_T) _uquad;
				cp = smerr2txt(err);
				if (cp == NULL)
					cp = "unknown_error";
				size = strlen(cp);
				VFP_PRINT(cp, size);

				cp = ", module=";
				size = strlen(cp);
				VFP_PRINT(cp, size);
				cp = smmod2txt(err);
				if (cp == NULL)
					cp = "unknown_module";
				size = strlen(cp);

				/* module/type?? */
				goto prtstring;
			}
			else { /* if (prec >= 0) */
				sm_error_T err;
				char *p;

				/*
				**  this is a hack...
				**  it doesn't deal with prec/width/... properly
				**  just don't use those with %M
				*/

				err = (sm_error_T) _uquad;
				cp = smerr2txt(err);
				if (cp == NULL)
					cp = "unknown_error";
				realsz = strlen(cp);
				if (realsz <= prec) {
					VFP_PRINT(cp, realsz);
					prec -= realsz;
					cp = ", module=";
					size = strlen(cp);
					if (size >= prec) {
						prec = 0;
						ret += realsz;
						goto flushit;
					}
					VFP_PRINT(cp, size);
					prec -= size;
					cp = smmod2txt(err);
					if (cp == NULL)
						cp = "unknown_module";
					size = strlen(cp);

					/* module/type?? */
					goto prtstring;
				}
				else {
					/* not enough space */
					p = memchr(cp, 0, prec);
					if (p != NULL) {
						size = p - cp;
						if (size > prec)
							size = prec;
					}
					else
						size = prec;
				}
			}
			sign = '\0';
			break;
		  case 'm':
			_uquad = SARG();
			if ((longlong_T) _uquad >= 0) {
#if COND_PRINT
				if (SLT_STATM == l) {
					VFP_PRINT("stat", strlen("stat"));
					VFP_PRINT("=", 1);
				}
#endif
				base = DEC;
				goto number;
			}
#if COND_PRINT
			if (SLT_STATM == l) {
				VFP_PRINT(name, strlen(name));
				VFP_PRINT("=", 1);
			}
#endif
			cp = smerr2txt((sm_ret_T) _uquad);
			if (cp == NULL || strcmp(cp, SM_ETXT_UNKNOWN) == 0) {
				_uquad &= (ulonglong_T) UINT32_MAX;
				base = HEX;
				xdigs = "0123456789abcdef";
				ch = 'x';
				flags |= HEXPREFIX;
				goto nosign;
			}
			goto prtstring;
		  case 'w':
			_uquad = SARG();
			cp = _uquad ? "true" : "false";
			goto prtstring;
		  case 's':
			if ((cp = GETARG(char *)) == NULL)
				cp = "(null)";
			else if (flags & ALT)
				flags |= SANITIZE;
   prtstring:
			if (prec >= 0) {
				/*
				**  can't use strlen; can only look for the
				**  NUL in the first `prec' characters, and
				**  strlen() will go further.
				*/

				char *p = memchr(cp, 0, prec);

				if (p != NULL) {
					size = p - cp;
					if (size > prec)
						size = prec;
				}
				else
					size = prec;
			}
			else
				size = strlen(cp);
			SANITIZE_SIZE(encode);
			sign = '\0';
			break;

		  case 'a':
			{
			uint32_t u32;
			uchar *src;
			sm_sockaddr_T *addr;

			addr = GETARG(sm_sockaddr_T *);
			UNSPEC_ADDR(addr);
			SM_PRT_IPV6(addr->sa.sa_family, addr->sin6.sin6_addr);
			SM_PRT_UNIX(addr->sa.sa_family, addr);
			SM_CHKFAM(addr->sa.sa_family);
			SM_PRT_IPV4(addr->sin.sin_addr.s_addr);
			}
			break;

		  case 'I':
			{
			uint32_t u32;
			uchar *src;
			sm_ipaddr_T *addr;

			addr = GETARG(sm_ipaddr_T *);
			UNSPEC_ADDR(addr);
			SM_PRT_IPV6(addr->ipa_family, addr->ipa_addr.ipv6_addr);
			if (AF_UNSPEC == addr->ipa_family && 0 == addr->ipa_addr.ipv4_addr)
			{
				(void) strlcpy(buf, "0.0.0.0", sizeof(buf));
				size = strlen(buf);
				cp = buf;
				break;
			}
			SM_CHKFAM(addr->ipa_family);
			SM_PRT_IPV4(addr->ipa_addr.ipv4_addr);
			}
			break;

		  case 'A':
			{
			uchar *src;
			uint32_t u32;

			_uquad = UARG();
			SM_PRT_IPV4(_uquad);
			}
			break;

#if MTA_NETINET6
		  case 'B':
			{
			ipv6_T *addr;

			addr = GETARG(ipv6_T *);
			SM_PRT_IPV6(AF_INET6, *addr);
			}
			break;
#endif

#if !SM_NO_CSTR
		  case 'C':
			size = 6; /* strlen("NULL") */
			if ((cstr = GETARG(sm_cstr_P)) == NULL)
				cp = "(NULL)";
			else {
				SM_IS_CSTR(cstr);
				if ((cp = (char *) sm_cstr_data(cstr)) == NULL)
					cp = "(Null)";
				else {
					size = sm_cstr_getlen(cstr);
					while (size > 0 && sm_cstr_rd_elem(cstr, size - 1) == '\0')
						--size;
					if (flags & ALT) {
						flags |= SANITIZE;
						SANITIZE_SIZE(encode);
					}
				}
			}
			if (prec >= 0 && size > prec)
				size = prec;
			sign = '\0';
			break;
#endif /* !SM_NO_CSTR */

		  case 'S':
			size = 6; /* strlen("NULL") */
			if ((str = GETARG(sm_str_P)) == NULL)
				cp = "(NULL)";
			else {
				SM_IS_BUF(str);
				if ((cp = (char *) sm_str_data(str)) == NULL)
					cp = "(Null)";
				else {
					size = sm_str_getlen(str);
					if (flags & ALT) {
						flags |= SANITIZE;
						SANITIZE_SIZE(encode);
					}
				}
			}
			if (prec >= 0 && size > prec)
				size = prec;
			sign = '\0';
			break;

		  case 'T':
			size = 6; /* strlen("NULL") */
			if ((str = GETARG(sm_str_P)) == NULL)
				cp = "(NULL)";
			else {
				SM_IS_BUF(str);
				if ((cp = (char *) sm_str_data(str)) == NULL)
					cp = "(Null)";
				else {
					uchar c;

					size = sm_str_getlen(str);
					while (size > 0 && (
					       (c = sm_str_rd_elem(str, size - 1)) == '\0' ||
					       c == '\n' || c == '\r'))
					{
						--size;
					}
				}
				if (flags & ALT) {
					flags |= SANITIZE;
					SANITIZE_SIZE(encode);
				}
			}
			if (prec >= 0 && size > prec)
				size = prec;
			sign = '\0';
			break;

		  case 'N':
			size = 6; /* strlen("NULL") */
			if ((str = GETARG(sm_str_P)) == NULL)
				cp = "(NULL)";
			else {
				SM_IS_BUF(str);
				if ((cp = (char *) sm_str_data(str)) == NULL)
					cp = "(Null)";
				else {
					size = sm_str_getlen(str);
					while (size > 0 && sm_str_rd_elem(str, size - 1) == '\0')
						--size;
				}
				if (flags & ALT) {
					flags |= SANITIZE;
					SANITIZE_SIZE(encode);
				}
			}
			if (prec >= 0 && size > prec)
				size = prec;
			sign = '\0';
			break;

		  case 'c':
			*buf = GETARG(int);
			cp = buf;
			size = 1;
			sign = '\0';
			if (flags & ALT) {
				flags |= SANITIZE;
				SANITIZE_SIZE(encode);
			}
			break;
		  case 'v':	/* time_t */
#if SIZEOF_TIME_T == 8
			flags |= LONGINT;
#elif SIZEOF_TIME_T == 4
#else
  ERROR _SIZEOF_TIME_T is neither 4 not 8: SIZEOF_TIME_T
#endif
			goto printint;
			break;

		  case 'D':
			flags |= LONGINT;
			/*FALLTHROUGH*/
		  case 'd':
		  case 'i':
  printint:
			_uquad = SARG();
			if ((longlong_T) _uquad < 0) {
				_uquad = -(longlong_T) _uquad;
				sign = '-';
			}
			base = DEC;
			goto number;

		  case 'U':
			flags |= LONGINT;
			/*FALLTHROUGH*/
		  case 'u':
			_uquad = UARG();
			base = DEC;
			goto nosign;

		  case 'r':
			_uquad = SARG();
			if ((longlong_T) _uquad < 0) {
				_uquad &= (ulonglong_T) UINT32_MAX;
				base = HEX;
				xdigs = "0123456789abcdef";
				ch = 'x';
				flags |= HEXPREFIX;
				goto nosign;
			}
			base = DEC;
			goto number;

		  case 'y':
			if ((cp = GETARG(char *)) == NULL || prec <= 0) {
				cp = "(null)";
				size = strlen(cp);
				goto prtstring;
			}
			if (flags & CHARINT)
				SM_PRT_HEX8(cp, prec, flags & ZEROPAD);
			else /* if (flags & SHORTINT) */
				SM_PRT_HEX16(cp, prec, flags & ZEROPAD);
#if 0
			else /* if (flags & LONGINT) */
				SM_PRT_HEX32(cp, prec, flags & ZEROPAD);
#endif
			break;

		  case 'Y':
			_uquad = UARG();
			base = BASE64;
			sign = '\0';
			goto number;

		  case 'Z':
			_uquad = UARG();
			base = BASE32;
			sign = '\0';
			goto number;

		  case 'X':
			xdigs = "0123456789ABCDEF";
			goto hex;
		  case 'x':
			xdigs = "0123456789abcdef";
hex:		_uquad = UARG();
			base = HEX;
			/* leading 0x/X only if non-zero */
			if (flags & ALT && _uquad != 0)
				flags |= HEXPREFIX;

			/* unsigned conversions */
nosign:		sign = '\0';

			/*
			**  ``... diouXx conversions ... if a precision is
			**  specified, the 0 flag will be ignored.''
			**	-- ANSI X3J11
			*/

number:		if ((dprec = prec) >= 0)
			flags &= ~ZEROPAD;

			/*
			**  ``The result of converting a zero value with an
			**  explicit precision of zero is no characters.''
			**	-- ANSI X3J11
			*/

			wcp = buf + BUF;	/* sizeof(buf)? */
			if (_uquad != 0 || prec != 0) {
				/*
				**  Unsigned mod is hard, and unsigned mod
				**  by a constant is easier than that by
				**  a variable; hence this switch.
				*/

				switch (base) {
				  case OCT:
					do {
						*--wcp = to_char(_uquad & 7);
						_uquad >>= 3;
					} while (_uquad);
					/* handle octal leading 0 */
					if (flags & ALT && *wcp != '0')
						*--wcp = '0';
					break;

				  case DEC:
					/* many numbers are 1 digit */
					while (_uquad >= 10) {
						*--wcp = to_char(_uquad % 10);
						_uquad /= 10;
					}
					*--wcp = to_char(_uquad);
					break;

				  case HEX:
					do {
						*--wcp = xdigs[_uquad & 15];
						_uquad >>= 4;
					} while (_uquad);
					break;

				  case BASE32:
					while (_uquad >= 32) {
						*--wcp = b32digs[_uquad & 31];
						_uquad >>= 5;
					}
					*--wcp = b32digs[_uquad & 31];
					break;

				  case BASE64:
					while (_uquad > 63) {
						*--wcp = b64digs[_uquad & 63];
						_uquad >>= 6;
					}
					*--wcp = b64digs[_uquad & 63];
					break;

				  default:
					cp = "bug in sm_io_vfprintf: bad base";
					size = strlen(cp);
					goto skipsize;
				}
			}
			size = buf + BUF - wcp;	/* sizeof(buf)? */
			cp = wcp;

		  skipsize:
			break;
