/*
 */

#include "sm/generic.h"
SM_RCSID("@(#)$Id: rcbgetsock.c,v 1.2 2011/01/30 21:49:06 ca Exp $")
#include "sm/limits.h"
#include "sm/assert.h"
#include "sm/magic.h"
#include "sm/memops.h"
#include "sm/rpool.h"
#include "sm/str-int.h"
#include "sm/rcb.h"
#include "sm/net.h"

/*
**  SM_RCB_GETSOCK -- Read data from rcb into sockaddr
**
**	Parameters:
**		rcb -- sm_rcb_P object to read from.
**		sockaddr -- sockaddr to fill.
**
**	Returns:
**		usual sm_error code; SM_E_OVFLW_NS
*/

sm_ret_T
sm_rcb_getsock(sm_rcb_P rcb, sm_sockaddr_P sockaddr)
{
	sm_ret_T ret;
	int len;
	uint32_t family, port;
	uchar *buf;

	SM_IS_RCB(rcb);
#if SM_RCB_CHECK
	SM_REQUIRE(rcb->sm_rcb_state == SM_RCB_DEC);
#endif
#if PARANOID
	SM_REQUIRE(rcb->sm_rcb_base != NULL);
#endif
	SM_REQUIRE(sockaddr != NULL);

	ret = sm_rcb_get2uint32(rcb, &family, &port);
	if (sm_is_err(ret))
		goto error;

	sockaddr->sa.sa_family = family;
	switch (sockaddr->sa.sa_family) {
	  case AF_INET:
		len = sizeof(sockaddr->sin.sin_addr);
		buf = (uchar *)&sockaddr->sin.sin_addr;
		sockaddr->sin.sin_port = port;
		break;

#if MTA_NETINET6
	  case AF_INET6:
		len = sizeof(sockaddr->sin6.sin6_addr);
		buf = (uchar *)&sockaddr->sin6.sin6_addr;
		sockaddr->sin6.sin6_port = port;
		break;
#endif
	  default:
		ret = sm_error_perm(SM_EM_NONE, EPROTONOSUPPORT);
		goto error;
	}
	ret = sm_rcb_getn(rcb, buf, len);
	if (sm_is_err(ret))
		goto error;

	return SM_SUCCESS;

  error:
	return ret;
}
