/*
 */

#include "sm/generic.h"
SM_RCSID("@(#)$Id: sm_sockaddr2a.c,v 1.4 2014/03/01 17:07:18 ca Exp $")

#include "sm/assert.h"
#include "sm/error.h"
#include "sm/ctype.h"
#include "sm/net.h"
#include "sm/str.h"
#if MTA_NETINET6 >= 2
#include "sm/memops.h"
#include "sm/nameser.h"
#endif

/*
**  SM_SOCKADDR2STR -- convert sockaddr to an ASCII string
**
**	Parameters:
**		sockaddr -- sockaddr struct
**		dst -- append ASCII representation of address (output)
**
**	Returns:
**		error: ENOSPC
*/

sm_ret_T
sm_sockaddr2str(sm_sockaddr_P sockaddr, sm_str_P dst)
{
	SM_REQUIRE(sockaddr != NULL);

	switch (sockaddr->sa.sa_family) {
	  case AF_INET:
		return sm_inet_inaddr2str(sockaddr->sin.sin_addr, dst);
		break;

#if MTA_NETINET6
	  case AF_INET6:
# if MTA_NETINET6 >= 2
		if (IN6_IS_ADDR_V4MAPPED(&sockaddr->sin6.sin6_addr)) {
			uchar *src;
			ipv4_T ipv4;

			src = &sockaddr->sin6.sin6_addr.s6_addr;
			sm_memcpy(&ipv4, src + IN6ADDRSZ - INADDRSZ, sizeof(ipv4));
			return sm_inet_ipv4str(ipv4, dst);
		}
# endif
		return sm_inet_in6faddrstr(sockaddr->sin6.sin6_addr, dst);
		break;
#endif
	  default:
		return sm_error_perm(SM_EM_NONE, EPROTONOSUPPORT);
	}

	/* NOTREACHED */
	return sm_error_perm(SM_EM_NONE, EINVAL);
}
