/*
 * Copyright (c) 2005 Sendmail, Inc. and its suppliers.
 *	All rights reserved.
 *
 * By using this file, you agree to the terms and conditions set
 * forth in the LICENSE file which can be found at the top level of
 * the sendmail distribution.
 */

#include "sm/generic.h"
SM_RCSID("@(#)$Id: sockconn.c,v 1.3 2010/03/02 05:21:04 ca Exp $")
#include "sm/assert.h"
#include "sm/error.h"
#include "sm/memops.h"
#include "sm/fcntl.h"
#include "sm/net.h"
#include "sm/sockcnf.h"
#include "sm/socket.h"

/*
**  SOCK_CONNECT -- Create a socket and connect it to a server.
**
**	Parameters:
**		sock -- address to bind to.
**		flags -- flags
**		pfd -- (pointer to) file descriptor (output)
**
**	Returns:
**		usual error code
*/

sm_ret_T
sock_connect(sockspec_P sock, uint flags, int *pfd)
{
	sm_ret_T ret;
	int sockfd;
	sockaddr_len_T sock_size;
	sm_sockaddr_T sm_sockaddr;

	SM_ASSERT(sock != NULL);
	SM_ASSERT(pfd != NULL);

	sockfd = INVALID_SOCKET;
	ret = sockspec2sock(sock, &sm_sockaddr, &sock_size, flags);
	if (sm_is_err(ret))
		goto error;

	sockfd = socket(sm_sockaddr.sa.sa_family, SOCK_STREAM, 0);
	if (sockfd == INVALID_SOCKET)
		return sm_error_perm(SM_EM_NET, errno);

	if ((connect(sockfd, (struct sockaddr *) &(sm_sockaddr.sa), sock_size))
		== -1)
	{
		ret = sm_error_perm(SM_EM_NET, errno);
		goto error;
	}
	*pfd = sockfd;
	return SM_SUCCESS;

  error:
	if (sockfd != INVALID_SOCKET)
		closesocket(sockfd);
	return ret;
}
