/*
 * Copyright (c) 2004, 2005 Sendmail, Inc. and its suppliers.
 *	All rights reserved.
 *
 * By using this file, you agree to the terms and conditions set
 * forth in the LICENSE file which can be found at the top level of
 * the sendmail distribution.
 */

#include "sm/generic.h"
SM_RCSID("@(#)$Id: strcasecmpoff.c,v 1.1 2013/10/01 02:02:25 ca Exp $")

#include "sm/assert.h"
#include "sm/magic.h"
#include "sm/memops.h"
#include "sm/str.h"
#include "sm/str-int.h"

/*
**  SM_STR_CASECMP_OFF -- Compare two strings (case insensitive).
**
**	Parameters:
**		s1 -- first string.
**		o1 -- offset in first string.
**		s2 -- second string.
**		o2 -- offset in second string.
**
**	Returns:
**		0: strings are equal
**		<0: 1. string < 2. string
**		>0: 1. string > 2. string
*/

int
sm_str_casecmp_off(const sm_str_P s1, uint o1, const sm_str_P s2, uint o2)
{
	int d, r;
	uint l;

	SM_IS_BUF(s1);
	SM_IS_BUF(s2);
	SM_ASSERT(o1 <= s1->sm_str_len);
	SM_ASSERT(o2 <= s2->sm_str_len);
	l = s1->sm_str_len - o1;
	if (l > s2->sm_str_len - o2) {
		d = 1;
		l = s2->sm_str_len - o2;
	}
	else if (l == s2->sm_str_len - o2)
		d = 0;
	else
		d = -1;
	r = strncasecmp((const char *) s1->sm_str_base + o1,
			(const char *) s2->sm_str_base + o2, l);
	if (r == 0)
		r = d;
	return r;
}
