/*
 */

#include "sm/generic.h"
SM_RCSID("@(#)$Id: t-pm-rcpts.c,v 1.4 2014/03/25 18:42:08 ca Exp $")
#include "sm/assert.h"
#include "sm/error.h"
#include "sm/heap.h"
#include "t-pmilter.h"

#if MTA_USE_PMILTER

/*
**  PMT_RCPT_NEW -- add a new recipient to the recipient list
**
**	Parameters:
**		pmtse_ctx -- SMTP session context
**		rcpt_pa -- recipient (printable address, RFC 2821)
**		ppmt_rcpt -- new recipient (output)
**
**	Returns:
**		usual return code
*/

sm_ret_T
pmt_rcpt_new(pmtse_ctx_P pmtse_ctx, const char *rcpt_pa, pmt_rcpt_P *ppmt_rcpt)
{
	pmt_rcpt_P pmt_rcpt;

	SM_REQUIRE(ppmt_rcpt != NULL);
	pmt_rcpt = (pmt_rcpt_P) sm_zalloc(sizeof(**ppmt_rcpt));
	if (pmt_rcpt == NULL)
		goto error;
	pmt_rcpt->pmtr_pa = sm_rpool_strdup(NULL, rcpt_pa);
	if (NULL == pmt_rcpt->pmtr_pa)
		goto error;
	PMT_RCPTS_INSERT_TAIL(&pmtse_ctx->pmtse_rcpts, pmt_rcpt);
	*ppmt_rcpt = pmt_rcpt;
	return SM_SUCCESS;

  error:
	SM_FREE(*ppmt_rcpt);
	return sm_err_temp(ENOMEM);
}

/*
**  PMT_RCPT_FREE -- free a single recipient address
**
**	Parameters:
**		pmtr_rcpt -- recipient to free
**
**	Returns:
**		usual return code
*/

static sm_ret_T
pmt_rcpt_free(pmt_rcpt_P pmt_rcpt)
{
	if (NULL == pmt_rcpt)
		return SM_SUCCESS;
	SM_FREE(pmt_rcpt->pmtr_pa);
	sm_free(pmt_rcpt);
	return SM_SUCCESS;
}

/*
**  PMT_RCPTS_FREE -- free an entire recipient list
**
**	Parameters:
**		pmtse_ctx -- SMTP session context
**
**	Returns:
**		usual return code
*/

sm_ret_T
pmt_rcpts_free(pmtse_ctx_P pmtse_ctx)
{
	pmt_rcpt_P pmt_rcpt, pmt_rcpt_nxt;

	if (PMT_RCPTS_EMPTY(&pmtse_ctx->pmtse_rcpts))
		return SM_SUCCESS;
	for (pmt_rcpt = PMT_RCPTS_FIRST(&pmtse_ctx->pmtse_rcpts);
	     pmt_rcpt != PMT_RCPTS_END(&pmtse_ctx->pmtse_rcpts);
	     pmt_rcpt = pmt_rcpt_nxt)
	{
		pmt_rcpt_nxt = PMT_RCPTS_NEXT(pmt_rcpt);

		/* remove addr from list? use PMT_RCPTS_REMOVE_FREE()? */
		pmt_rcpt_free(pmt_rcpt);
	}
	PMT_RCPTS_INIT(&pmtse_ctx->pmtse_rcpts);
	return SM_SUCCESS;
}

/*
**  PMT_RCPTS_INIT -- init recipient list
**
**	Parameters:
**		pmtse_ctx -- SMTP session context
**
**	Returns:
**		usual return code
*/

sm_ret_T
pmt_rcpts_init(pmtse_ctx_P pmtse_ctx)
{
	PMT_RCPTS_INIT(&pmtse_ctx->pmtse_rcpts);
	return SM_SUCCESS;
}
#endif /* MTA_USE_PMILTER */
